package com.trendify.service;

import com.trendify.entity.Cart;
import com.trendify.entity.CartItem;
import io.github.resilience4j.circuitbreaker.annotation.CircuitBreaker;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import com.trendify.repo.CartRepository;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class CartService {

    @Autowired
    private  CartRepository cartRepository;

    @Autowired
    @Lazy
    private  WebClient.Builder webClientBuilder;

    private static final String PRODUCT_SERVICE_URL = "http://localhost:8081/products/product/price/";
    private static final String INVENTORY_SERVICE_URL = "http://localhost:8083/inventory/quantity/";


    public Mono<Cart> getCart(String userId) {
        return cartRepository.findByUserId(userId)
                .switchIfEmpty(Mono.just(createNewCart(userId)));
    }

    public Flux<Cart> getAll(){
        return cartRepository.findAll();
    }

    public Mono<Cart> addItemToCart(String userId, CartItem item) {
        return getCart(userId)
                .flatMap(cart -> getInventoryQuantity(item.getProductId())
                        .flatMap(inventoryQuantity -> {
                            boolean isAvailable = item.getQuantity() <= inventoryQuantity;
                            String newStatus = isAvailable ? "available" : "out of stock";

                            return getProductPrice(item.getProductId())
                                    .flatMap(productPrice -> {
                                        List<CartItem> items = cart.getItems();
                                        if (items == null) {
                                            items = new ArrayList<>();
                                        }

                                        Optional<CartItem> existingItemOptional = items.stream()
                                                .filter(i -> i.getProductId().equals(item.getProductId()))
                                                .findFirst();

                                        if (existingItemOptional.isPresent()) {
                                            CartItem existingItem = existingItemOptional.get();
                                            if (existingItem.getStatus().equals("out of stock") && isAvailable) {
                                                existingItem.setQuantity(item.getQuantity());
                                                existingItem.setStatus("available");
                                                existingItem.setPrice(productPrice * item.getQuantity());
                                            } else {
                                                existingItem.setQuantity(existingItem.getQuantity() + item.getQuantity());
                                                existingItem.setStatus(newStatus);
                                                if ("available".equals(newStatus)) {
                                                    existingItem.setPrice(productPrice * existingItem.getQuantity());
                                                } else {
                                                    existingItem.setPrice(0.0);
                                                }
                                            }
                                        } else {
                                            item.setStatus(newStatus);
                                            if("out of stock".equals(newStatus)){
                                                item.setPrice(0.0);
                                            } else {
                                                item.setPrice(productPrice * item.getQuantity());
                                            }
                                            items.add(item);
                                        }

                                        cart.setItems(items);
                                        cart.setTotalPrice(calculateTotalPrice(items));
                                        return cartRepository.save(cart);
                                    });
                                    //.onErrorResume(throwable -> Mono.error(throwable)); // propagate exception
                                    /*.onErrorResume(e -> {
                                        item.setPrice(0.0);
                                        item.setStatus("out of stock");

                                        List<CartItem> items = cart.getItems();
                                        if (items == null) {
                                            items = new ArrayList<>();
                                        }

                                        Optional<CartItem> existingItemOptional = items.stream()
                                                .filter(i -> i.getProductId().equals(item.getProductId()))
                                                .findFirst();

                                        if (existingItemOptional.isPresent()) {
                                            CartItem existingItem = existingItemOptional.get();
                                            existingItem.setQuantity(existingItem.getQuantity() + item.getQuantity());
                                            existingItem.setPrice(0.0);
                                        } else {
                                            items.add(item);
                                        }

                                        cart.setItems(items);
                                        cart.setTotalPrice(calculateTotalPrice(items));
                                        return cartRepository.save(cart);
                                    });*/
                        })
                        //.onErrorResume(throwable -> Mono.error(throwable)) // propagate exception
                        .switchIfEmpty(Mono.defer(() -> {
                            item.setPrice(0.0);
                            item.setStatus("out of stock");

                            return getProductPrice(item.getProductId())
                                    .flatMap(productPrice -> {
                                        List<CartItem> items = cart.getItems();
                                        if (items == null) {
                                            items = new ArrayList<>();
                                        }

                                        Optional<CartItem> existingItemOptional = items.stream()
                                                .filter(i -> i.getProductId().equals(item.getProductId()))
                                                .findFirst();

                                        if (existingItemOptional.isPresent()) {
                                            CartItem existingItem = existingItemOptional.get();
                                            existingItem.setQuantity(existingItem.getQuantity() + item.getQuantity());
                                            existingItem.setPrice(0.0);
                                        } else {
                                            items.add(item);
                                        }

                                        cart.setItems(items);
                                        cart.setTotalPrice(calculateTotalPrice(items));
                                        return cartRepository.save(cart);
                                    })
                                    .onErrorResume(ResponseStatusException.class, e -> Mono.error(e)); // propagate exception
                                   /* .onErrorResume(e -> {
                                        List<CartItem> items = cart.getItems();
                                        if (items == null) {
                                            items = new ArrayList<>();
                                        }

                                        Optional<CartItem> existingItemOptional = items.stream()
                                                .filter(i -> i.getProductId().equals(item.getProductId()))
                                                .findFirst();

                                        if (existingItemOptional.isPresent()) {
                                            CartItem existingItem = existingItemOptional.get();
                                            existingItem.setQuantity(existingItem.get().getQuantity() + item.getQuantity());
                                            existingItem.setPrice(0.0);
                                        } else {
                                            items.add(item);
                                        }

                                        cart.setItems(items);
                                        cart.setTotalPrice(calculateTotalPrice(items));
                                        return cartRepository.save(cart);
                                    });*/
                        })));
    }


    private Cart createNewCart(String userId) {
        Cart cart = new Cart();
        cart.setUserId(userId);
        return cart;
    }


    public Mono<Cart> removeItemFromCart(String userId, String productId) {
        return getCart(userId)
                .flatMap(cart -> {
                    List<CartItem> updatedItems = cart.getItems().stream()
                            .filter(item -> !item.getProductId().equals(productId))
                            .collect(Collectors.toList());

                    cart.setItems(updatedItems);
                    return cartRepository.save(cart);
                });
    }
    @CircuitBreaker(name = "productService", fallbackMethod = "getProductPriceFallback")
    public Mono<Double> getProductPrice(String productId) {

        //throw new RuntimeException("Forced exception for testing");
        return webClientBuilder.build()
                .get()
                .uri(PRODUCT_SERVICE_URL + productId)
                .retrieve()
                .onStatus(httpStatus -> httpStatus.isError(), response ->
                        response.bodyToMono(String.class)
                                .flatMap(errorBody -> Mono.error(new RuntimeException("Product service error: " + errorBody)))
                )
                .bodyToMono(Double.class);
                //.onErrorResume(throwable -> Mono.error(throwable)); // Ensure proper propagation
    }
    @CircuitBreaker(name = "inventoryService", fallbackMethod = "getInventoryQuantityFallback")
    public Mono<Integer> getInventoryQuantity(String productId) {
        return webClientBuilder.build()
                .get()
                .uri(INVENTORY_SERVICE_URL + productId)
                .retrieve()
                .onStatus(httpStatus -> httpStatus.isError(), response ->
                        response.bodyToMono(String.class)
                                .flatMap(errorBody -> Mono.error(new RuntimeException("Product service error: " + errorBody)))
                )
                .bodyToMono(Integer.class);
    }

    private Mono<Double> getProductPriceFallback(String productId, Throwable t) {
        // Handle fallback logic for product price (e.g., return a default value)
        System.err.println("getProductPriceFallback called. Error: " + t.getMessage());
        return Mono.error(new ResponseStatusException(HttpStatus.SERVICE_UNAVAILABLE, "Product service is unavailable."));
    }

    private Mono<Integer> getInventoryQuantityFallback(String productId, Throwable t) {
        // Handle fallback logic for inventory quantity (e.g., return a default value)
        System.err.println("getInventoryQuantityFallback called. Error: " + t.getMessage());
        return Mono.error(new ResponseStatusException(HttpStatus.SERVICE_UNAVAILABLE, "Inventory service is unavailable."));
    }

    private double calculateTotalPrice(List<CartItem> items) {
        if(items == null || items.isEmpty()){
            return 0.0;
        }
        return items.stream()
                .mapToDouble(CartItem::getPrice)
                .sum();
    }


}
